class InvalidArgumentException(Exception):
    pass


class InvalidChiaLocationException(Exception):
    pass


class InvalidConfigurationSetting(Exception):
    pass


class InvalidYAMLConfigException(Exception):
    pass


class ManagerError(Exception):
    pass


class MissingImportError(Exception):
    pass


class TerminationException(Exception):
    pass


class TerminalTooSmallError(Exception):
    pass
